--*******************************************************************************************************************************
--
-- Description: This module performs read and write operations to the two QDR II+ SRAM chips as commanded over the host bus.
--				Note: QDR II+ SRAM-A is assigned to index 0 and QDR II+ SRAM-B is assigned to index 1 in this example.
--					
-- Rev History: 10-07-2013 Initial Release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all; 

Library UNISIM; 
use UNISIM.Vcomponents.all;

use work.model_specific_package.all;
use work.transceiver_package.all;

entity qdr_sram is 
	port (
	--  Reset and clocks supplied by the application logic.
	rst         		: in std_logic;		-- Active high asynchronous reset.
	clk 				: in std_logic;		-- Bus clock.
	clk200 				: in std_logic;		-- 200 MHz clock on global clock buffer.
	qdriip_sys_clk_p 	: in std_logic;		-- System clock to both QDR II+ SRAM controllers.
	qdriip_sys_clk_n 	: in std_logic;		-- Differential pair (see above).
	
	-- QDR II+ SRAM controller interface signals.
	qdr_wr_data			: in DUAL_72;		-- Data bus carrying four 18-bit QDR II+ SRAM write data values.
	qdr_bw_n			: in DUAL_8;		-- Eight active low byte enables assigned to the QDR II+ SRAM write data values.
	qdr_rd_data			: out DUAL_72;		-- Data bus carrying four 18-bit QDR II+ SRAM read data values.
	qdr_addr			: in DUAL_QDR;		-- Address selected for current operation.
	qdr_cmd				: in DUAL_2;		-- Single period write (0) and read (1) command strobes in the PCI clock domain.
	qdr_rd_valid		: out DUAL_1;		-- Logic one indicates that read data is available from the QDR II+ SRAM.
	qdr_calib_complete	: out DUAL_1;		-- Logic one indicates that the controller has completed timing calibration.
	
	--	The following signals connect to ADC devices through FPGA pins.
	qdriip_cq			: in  DUAL_1;		-- QDR II+ SRAM echo clock.
	qdriip_q			: in  DUAL_18;		-- QDR II+ SRAM synchronous data outputs.
	qdriip_qvld			: in  DUAL_1;		-- QDR II+ SRAM read data valid flag.
	qdriip_k_p			: out DUAL_1;		-- QDR II+ SRAM input clock.
	qdriip_k_n			: out DUAL_1;		-- QDR II+ SRAM differential pair (see above).
	qdriip_d			: out DUAL_18;		-- QDR II+ SRAM synchronous data inputs.
	qdriip_sa			: out DUAL_QDR;		-- QDR II+ SRAM synchronous address inputs.
	qdriip_w_n			: out DUAL_1;		-- QDR II+ SRAM synchronous active low write enable.
	qdriip_r_n			: out DUAL_1;		-- QDR II+ SRAM synchronous active low read enable.
	qdriip_bw_n			: out DUAL_2;		-- QDR II+ SRAM synchronous byte enables.
	qdriip_dll_off_n	: out DUAL_1		-- QDR II+ SRAM active low DLL disable.
	);	
end qdr_sram;

architecture behavioral of qdr_sram is

	signal half_sys_clk	: std_logic;	-- Half frequency clock derived from the system clock.
	signal wr_cmd		: DUAL_1;		-- Single period write command strobe in the QDR II+ SRAM clock domain.
	signal rd_cmd		: DUAL_1;		-- Single period read command strobe in the QDR II+ SRAM clock domain.
	signal rd_data		: DUAL_72;		-- Data bus carrying four 18-bit QDR II+ SRAM read data values.
	signal read_data	: DUAL_72;		-- Data bus carrying four 18-bit QDR II+ SRAM read data values.
	signal rd_valid		: DUAL_1;		-- Logic one indicates that read data is available from the QDR II+ SRAM.
	signal read_valid	: DUAL_1;		-- Logic one indicates that read data is available from the QDR II+ SRAM.
	signal wr_data		: DUAL_72;		-- Data bus carrying four 18-bit QDR II+ SRAM write data values.
	signal bw_n			: DUAL_8;		-- Eight active low byte enables assigned to the QDR II+ SRAM write data values.
	signal addr			: DUAL_QDR;		-- Address selected for current operation.
	
begin
 	
--*************************************************** CLOCK DOMAIN CROSSING ****************************************************

	-- Generate the same logic for both the SRAM-A (0) and SRAM-B (1) interfaces.
	SRAM_INDEX : for i in 0 to 1 generate
	
		-- Create a single period write enable in the 250 MHz QDR II+ SRAM interface clock domain.
		SRAM_WR : entity work.clock_crossing
		port map (
			rst					=> rst,
			source_clk			=> clk,
			target_clk  		=> half_sys_clk,
			source_to_target	=> qdr_cmd(i)(0),
			target_enable		=> wr_cmd(i)
		);
	
		-- Create a single period read enable in the 250 MHz QDR II+ SRAM interface clock domain.
		SRAM_RD : entity work.clock_crossing
		port map (
			rst					=> rst,
			source_clk			=> clk,
			target_clk  		=> half_sys_clk,
			source_to_target	=> qdr_cmd(i)(1),
			target_enable		=> rd_cmd(i)
		);

		-- Register the read data in the 250 MHz QDR II+ SRAM interface clock domain.
		process (half_sys_clk,rst)
		begin
			if rst='1' then
				read_data(i) <= (others => '0');
				read_valid(i) <= '0';
				wr_data(i) <= (others => '0');
				bw_n(i) <= (others => '0');
				addr(i) <= (others => '0');
			elsif rising_edge(half_sys_clk) then
				-- Clear the read valid status bit to the PCI interface when a new command is initiated.
				if rd_cmd(i) <= '1' then
					read_valid(i) <= '0';
				-- Wait for valid data on the read bus.
				elsif rd_valid(i) = '1' then
					-- Register valid read data.
					read_data(i) <= rd_data(i);
					-- Set the read valid status bit to the PCI interface.
					read_valid(i) <= '1';
				end if;
				-- Reclock BAR2 registers in the QDR II+ SRAM clock domain.
				wr_data(i) <= qdr_wr_data(i);
				bw_n(i) <= qdr_bw_n(i);
				addr(i) <= qdr_addr(i);
			end if;
		end process;
 			
		-- Reclock outputs from the SRAM interface in the PCIe clock domain.
		process (clk,rst)
		begin
			if rst='1' then
				qdr_rd_data(i) <= (others => '0');
				qdr_rd_valid(i) <= '0';
			elsif rising_edge(clk) then
				qdr_rd_data(i) <= read_data(i);
				qdr_rd_valid(i) <= read_valid(i);
			end if;
		end process;
 			
	end generate;

--************************************************ DUAL QDR II+ SRAM INTERFACE *************************************************

	DUAL_SRAM : entity work.dual_qdr_sram
	port map (
		rst                               => rst,
		clk200                            => clk200,
		sram_clk                          => half_sys_clk,
		sram_a_wr_cmd                     => wr_cmd(0),
		sram_a_wr_addr                    => addr(0),
		sram_a_wr_data                    => wr_data(0),
		sram_a_wr_bw_n                    => bw_n(0),
		sram_a_rd_cmd                     => rd_cmd(0),
		sram_a_rd_addr                    => addr(0),
		sram_a_rd_data                    => rd_data(0),
		sram_a_rd_valid                   => rd_valid(0),
		sram_a_calib_complete             => qdr_calib_complete(0),
		sram_a_rst_clk                    => open,
		sram_b_wr_cmd                     => wr_cmd(1),
		sram_b_wr_addr                    => addr(1),
		sram_b_wr_data                    => wr_data(1),
		sram_b_wr_bw_n                    => bw_n(1),
		sram_b_rd_cmd                     => rd_cmd(1),
		sram_b_rd_addr                    => addr(1),
		sram_b_rd_data                    => rd_data(1),
		sram_b_rd_valid                   => rd_valid(1),
		sram_b_calib_complete             => qdr_calib_complete(1),
		sram_b_rst_clk                    => open,
		qdriip_sys_clk_p                  => qdriip_sys_clk_p,
		qdriip_sys_clk_n                  => qdriip_sys_clk_n,
		qdriip_cq                         => qdriip_cq,
		qdriip_q                          => qdriip_q,
		qdriip_qvld                       => qdriip_qvld,
		qdriip_k_p                        => qdriip_k_p,
		qdriip_k_n                        => qdriip_k_n,
		qdriip_d                          => qdriip_d,
		qdriip_sa                         => qdriip_sa,
		qdriip_w_n                        => qdriip_w_n,
		qdriip_r_n                        => qdriip_r_n,
		qdriip_bw_n                       => qdriip_bw_n,
		qdriip_dll_off_n                  => qdriip_dll_off_n
	);
	
end behavioral;
